# フォーマット

フォーマットとは、日時からある形式に基づいた文字列を作成することです  

```
Koyomi.format('2015-12-23 12:34:56', 'Y年M月D日 Ajh時m分s秒'); // 2015年12月23日 午後0時34分56秒
```

指定できる形式はパラメータ文字列と呼ばれる部品を組み合わせて作成します  
メソッドはクラスメソッドとインスタンスメソッドの両方で使用できます  
ただし、使用できるパラメータ文字列に若干の違いが存在するため注意が必要です

定義

`{String} Koyomi.format({Date|String} date, {String} format);`  

インスタンスメソッドはクラスメソッドの上位互換になっています

  + date
    + 整形対象の日時です
  + format
    + 整形するためのフォーマットです
    + パラメータ文字列を組み合わせて表現します
    + 既定値
        + クラスメソッドは`YYYY-MM-DD HH:mm:ss`(CONFIG.FORMAT)の値です
        + インスタンスメソッドは`koyomi.defaultFormat`の値です

```
koyomi.format('2015-4-27', 'Y/M/D');  // '2015/4/27'
```

# パラメータ文字列

以下の文字が format パラメータ文字列として認識されます  
限定に〇が付いているものは、クラスメソッドでは使用できません

| 区分           | 文字     | 説明                             | 戻り値の例                      |  型  |限定| 
|:--------------:|----------|----------------------------------|---------------------------------|:----:|:--:|
| カスタマイズ   | WAREKI   | 同`GGN年M月D日>>漢数字`          | 平成元年一月七日                | 文字 |    |
|                | wareki   | 同`ggn年M月D日>>漢数字`          | 昭和六十四年一月七日            | 文字 |    |
|                |          |                                  |                                 |      |    |
| 年(西暦)       | YYYY     | 西暦 0埋め 4桁                   | 0980, 1974, 2001, 2014, 2020    | 文字 |    |
|                | Y        | 西暦                             | 980, 1974, 2001, 2014, 2020     | 数字 |    |
|                | y        | 西暦 下2桁                       | 80, 74, 01, 14, 20              | 文字 |    |
|                |          |                                  |                                 |      |    |
| 年(皇紀)       | GGG      | 紀元前660年を元年とする          | 2634, 2640, 2661, 2674, 2680    | 数字 |    |
|                |          |                                  |                                 |      |    |
| 年(和暦)       | GG       | 年号 1872年以前は西暦            | 平成, 昭和, 大正, 明治, 西暦    | 文字 |    |
|   年区切り     | G        | 年号 省略表示（アルファベット）  | H, S, T, M                      | 文字 |    |
|                | N        | 年                               | 1, 5, 8, 15, 27, 30, 105        | 数字 |    |
|                |          |                                  |                                 |      |    |
| 年(和暦)       | gg       | 年号 1872年以前は西暦            | 平成, 昭和, 大正, 明治, 西暦    | 文字 |    |
|  日付区切り    | g        | 年号 省略表示（アルファベット）  | H, S, T, M                      | 文字 |    |
|                | n        | 年                               | 1, 5, 8, 15, 27, 30, 105        | 数字 |    |
|                |          |                                  |                                 |      |    |
| 上半期・下半期 | VV       | 英語 上:1-6月、下:7-12月         | first half, second half         | 文字 |    |
|                | V        | 日本語 上:1-6月、下:7-12月       | 上半期, 下半期                  | 文字 |    |
|                | Vx       | 日本語 上半期の始まり:startMonth | 上半期, 下半期                  | 文字 | 〇 |
|                |          |                                  |                                 |      |    |
| 四半期         | Q        | 開始月:1月                       | 1, 2, 3, 4                      | 数字 |    |
|                | Qx       | 開始月:startMonth                | 1, 2, 3, 4                      | 数字 | 〇 |
|                |          |                                  |                                 |      |    |
| 月             | MMM      | 月 英語                          | January, February...December    | 文字 |    |
|                | MM       | 月 0埋め                         | 01, 02, 03, 04...09, 10, 11, 12 | 文字 |    |
|                | M        | 月                               | 1, 2, 3, 4...9, 10, 11, 12      | 数字 |    |
|                |          |                                  |                                 |      |    |
| 週番号         | R        | 開始月:1月、開始曜日:月          | 1, 2, 3, 4...51, 52, 53, 54     | 数字 |    |
|                | Rj       | 開始月:4月、開始曜日:日          | 1, 2, 3, 4...51, 52, 53, 54     | 数字 |    |
|                | Ri       | ISO週番号                        | 1, 2, 3, 4...51, 52, 53, 54     | 数字 |    |
|                | Rx       | startMonthとstartWeekに依存      | 1, 2, 3, 4...51, 52, 53, 54     | 数字 | 〇 |
|                |          |                                  |                                 |      |    |
| 日             | DD       | 日 0埋め                         | 05, 15, 25                      | 文字 |    |
|                | D        | 日                               | 5, 15, 25                       | 数字 |    |
|                |          |                                  |                                 |      |    |
| 経過日数(年)   | TT       | 総日数   (年) 開始月:1月         | 365, 366                        | 数字 |    |
|                | T        | 経過日数 (年) 開始月:1月         | 1, 2, 3...364, 365, 366         | 数字 |    |
|                | t        | 残日数   (年) 開始月:1月         | 1, 2, 3...364, 365, 366         | 数字 |    |
|                | TTx      | 総日数   (年) 開始月:startManth  | 365, 366                        | 数字 | 〇 |
|                | Tx       | 経過日数 (年) 開始月:startMonth  | 1, 2, 3...364, 365, 366         | 数字 | 〇 |
|                | tx       | 残日数   (年) 開始月:startMonth  | 1, 2, 3...364, 365, 366         | 数字 | 〇 |
|                |          |                                  |                                 |      |    |
| 経過日数(月)   | UU       | 総日数   (月)                    | 28, 29, 30, 31                  | 数字 |    |
|                | U        | 経過日数 (月)                    | 1, 2, 3...29, 30, 31            | 数字 |    |
|                | u        | 残日数   (月)                    | 1, 2, 3...29, 30, 31            | 数字 |    |
|                |          |                                  |                                 |      |    |
| 営業日数(年)   | BB       | 営業総日数(年) 開始月:startMonth | 250, 260                        | 数字 | 〇 |
|                | B        | 営業日番号(年) 開始月:startMonth | 1, 2, 3...366                   | 数字 | 〇 |
|                | b        | 営業残日数(年) 開始月:startMonth | 1, 2, 3...366                   | 数字 | 〇 |
|                |          |                                  |                                 |      |    |
| 営業日数(月)   | PP       | 営業総日数(月)                   | 20, 25, 30                      | 数字 | 〇 |
|                | P        | 営業経過日数(月)                 | 1, 2, 3...30, 31                | 数字 | 〇 |
|                | p        | 営業残日数(月)                   | 1, 2, 3...30, 31                | 数字 | 〇 |
|                |          |                                  |                                 |      |    |
| 曜日           | WW       | 英語                             | Sunday, Monday...Sutarday       | 文字 |    |
|                | W        | 数字 日:0   -> 土:6              | 0, 1, 2, 3, 4, 5, 6             | 文字 |    |
|                | Wj       | 日本語                           | 日曜日, 月曜日...金曜日, 土曜日 | 文字 |    |
|                |          |                                  |                                 |      |    |
| 祝日           | FFw      | 祝日名または曜日                 | 元日, 成人の日, 日曜日...土曜日 | 文字 |    |
|                | FF       | 祝日名                           | 元日, 成人の日, 春分の日        | 文字 |    |
|                | Fw       | 祝日または曜日                   | 祝日, 日曜日, 月曜日...土曜日   | 文字 |    |
|                | F        | 祝日                             | 祝日                            | 文字 |    |
|                |          |                                  |                                 |      |    |
| 営業・休業     | Ew       | 曜日または休業日                 | 日曜日, 月曜日...土曜日, 休業日 | 文字 | 〇 |
|                | E        | 営業日または休業日               | 営業日, 休業日                  | 文字 | 〇 |
|                | e        | 休業理由                         | 祝日, 休業日, 定休日            | 文字 | 〇 |
|                |          |                                  |                                 |      |    |
| 時             | HH       | 時 24時間制 0埋め                | 03, 09, 15                      | 文字 |    |
|                | H        | 時 24時間制                      | 3, 9, 15                        | 数字 |    |
|                | hh       | 時 12時間制 0埋め                | 03, 09                          | 文字 |    |
|                | h        | 時 12時間制                      | 3, 9                            | 数字 |    |
|                |          |                                  |                                 |      |    |
| 分             | mm       | 分 0埋め                         | 03, 09                          | 文字 |    |
|                | m        | 分                               | 3, 9                            | 数字 |    |
|                |          |                                  |                                 |      |    |
| 秒             | ss       | 秒 0埋め                         | 03, 09, 15                      | 文字 |    |
|                | s        | 秒                               | 3, 9, 15                        | 数字 |    |
|                |          |                                  |                                 |      |    |
| 午前午後       | AA       | AM/PM                            | AM, PM                          | 文字 |    |
|                | A        | am/pm                            | am, pm                          | 文字 |    |
|                | Aj       | 午前/午後                        | 午前, 午後                      | 文字 |    |
|                |          |                                  |                                 |      |    |
| タイムゾーン   | Z        | オフセット                       | +9:00, -2:00                    | 文字 |    |
|                |          |                                  |                                 |      |    |
| 年齢           | O        | 年齢                             | 10, 23, 41                      | 数字 |    |


# パラメータ文字列 (アルファベット別)

パラメータ文字列はある程度、意味を持った区分によりアルファベットが割り当ててあります  
覚える場合は、下記の表のほうが役に立つでしょう

|   |        区分    |                      意味                        |
|:-:|----------------|--------------------------------------------------|
| A | 午前午後       | AMPM                                             |
| B | 営業日数(年)   | Business                                         |
| C | (未使用)       |                                                  |
| D | 日             | Date                                             |
| E | 営業日・休業日 | 営業日(Eigyobi)のE                               |
| F | 祝日           | 祭日はfate day (祝日はHolidayだがHは使用済みの為)|
| G | 年号           | 年号とほぼ同等の意味の元号のG                    |
| H | 時             | hours                                            |
| I | (未使用)       |                                                  |
| J | 日本語         | 接尾語。日本語を表す。WjやAjなど                 |
| K | (未使用)       |                                                  |
| L | (未使用)       |                                                  |
| M | 月もしくは分   | 大文字はMonth, 小文字はminutes                   |
| N | 年(和暦)       | 年(Nen)のN                                       |
| O | 年齢           | years oldのo                                     |
| P | 営業日数(月)   | 形・発声がBに似ている、月区分                    |
| Q | 四半期         | Quarter                                          |
| R | 週番号         | カレンダーにすると行を表すため(Row)              |
| S | 秒             | Seconds                                          |
| T | 経過日数(年)   | 年(Toshi)の日数                                  |
| U | 経過日数(月)   | Tの次のため                                      |
| V | 上半期・下半期 | 半分に割る形状より                               |
| W | 曜日           | Week                                             |
| X | インスタンス用 | 接尾語。バツ（クラスでは使用できない意味）       |
| Y | 年             | Year                                             |
| Z | タイムゾーン   | time zone                                        |


# 0埋め、切り捨て、序数

## 0埋め

例えば5時7分9秒を`'H:m:s'`と`'HH:mm:ss'`でフォーマットした場合に`'5:7:9'`と`'05:07:09'`となります。  
明らかに0埋めをしている後者の方が時刻として理解しやすくなります。  
このように、0埋めをサポートしているパラメータ文字列は、他に月と日にあります。  
ただ他のパラメータ文字列でも0埋めを行いたい場合もあります。  
その場合は、パラメータ文字列の後ろに続けて`>`と2から9までの数字を記載してください。  
数値の前に指定桁になるまで0埋めを行います。  
先ほどの`'HH:mm:ss'`は`'H>2:m>2:s>2'`と同じ結果を返します。  
ただし、パラメータ文字列が返す型が数字の場合のみが対象です。

## 切り捨て

パラメータ文字列が数字ではなく文字列を返す場合、その後ろに`>`と1から9の数字を追加すると、
その結果から文字数以上を切り捨てます。  
例えば、金曜日の日付に`'WW>3'`のformatを使用すると`'Fri'`を返します

## 序数

序数とは1st, 2nd, 3rdでのst, nd, rdのことです  
序数を数値の後ろに追加したい場合は、`>`と数字の0を追加してください  
たとえば、2日に`'D>0'`のformatを使用すると`'2nd'`の結果を表示します  
序数はパラメータ文字列が数字にも文字列にも対応します  
文字列の場合は、数字にできる場合にのみ序数を追加します


# 省略、元年、漢数字、漢字、全角

整形後の文字列をさらに変換するにはformatの最後に`'>>オプション名'`を追加します

  + `'>>省略'`  : 0分と0秒を省略します    **8時0分** &#x226B; **8時**
  + `'>>元年'`  : 1年を元年に変更します
  + `'>>漢数字'`: 元年の処理と数字部分を漢数字にします **27** &#x226B; **二十七**
  + `'>>漢字'`  : 元年の処理と数字部分を漢字にします   **27** &#x226B; **二七**
  + `'>>全角'`  : 数字部分を全角にします   **27** &#x226B; **２７**
  + 省略とその他のオプションを同時に指定することができます `'>>省略漢数字'`
  + 変換の優先順位は必ず、元年、省略、漢数字、漢字、全角の順です

次のようにすると漢数字で表示します

```
koyomi.format('2015-4-27', 'GGN年M月D日>>漢数字'); // '平成二十七年四月二十七日'
```

# エスケープ

変換されたくない文字がたまたまパラメータ文字列だった場合は、次のようにしてください

```
Koyomi.format('2015-5-10', '{Y}/{M}/{D} updated'); // 2015/5/10 updated
```

カッコが一つでも存在すると、そこだけパラメータ文字列だと判断しその他の文字は変換されません


# ドキュメント一覧

  + [イントロダクション ../README.md](../README.md)
  + [インスタンスの作成 ./instance.md](./instance.md)
  + フォーマット ./format.md
  + [日時の情報取得・操作 ./calc-date.md](./calc-date.md)
  + [営業日計算 ./eigyobi.md](./eigyobi.md)
  + [カレンダー情報 ./calendar.md](./calendar.md)
  + [年度 ./nendo.md](./nendo.md)
  + [祝日 ./holiday.md](./holiday.md)
  + [営業・休業 ./open-close.md](./open-close.md)
  + [補助関数 ./helper.md](./helper.md)
